﻿using Borepin.Base;
using Prism.Commands;
using Prism.Navigation;
using System.Threading.Tasks;
using System.Windows.Input;
using FabAccessAPI.Schema;
using Borepin.Service.BFFH;
using static FabAccessAPI.Schema.MachineSystem;

namespace Borepin.PageModel
{
    public class MachinePageModel : PageModelBase
    {
        #region Private Properties
        private IBFFHService _BFFHService;
        private UUID _ID;
        #endregion

        #region Contructors
        public MachinePageModel(INavigationService navigationService, IBFFHService bffhService) : base(navigationService)
        {
            _BFFHService = bffhService;

            UseMachineCommand = new DelegateCommand(UseMachineCommandExecuted);
            GiveBackMachineCommand = new DelegateCommand(GiveBackMachineCommandExecuted);
        }
        #endregion

        #region Data
        public override async Task LoadData()
        {
            IMachineSystem machineSystem = await _BFFHService.GetMachineSystemInterface();

            IInfoInterface info = await machineSystem.Info();

            MachineItem = (await info.GetMachine(_ID)).Item1;

            IsBusy = false;
        }
        #endregion

        #region Properties
        private Machine _MachineItem;
        public Machine MachineItem
        {
            get => _MachineItem;
            set => SetProperty(ref _MachineItem, value);
        }
        #endregion

        #region Commands
        private ICommand _UseMachineCommand;

        public ICommand UseMachineCommand
        {
            get => _UseMachineCommand;
            set => SetProperty(ref _UseMachineCommand, value);
        }

        private async void UseMachineCommandExecuted()
        {
            Machine.IUseInterface useInterface = MachineItem.Use;

            await useInterface.Use();
        }

        private ICommand _GiveBackMachineCommand;
        public ICommand GiveBackMachineCommand
        {
            get => _GiveBackMachineCommand;
            set => SetProperty(ref _GiveBackMachineCommand, value);
        }

        private async void GiveBackMachineCommandExecuted()
        {
            Machine.IInUseInterface inUseInterface = MachineItem.Inuse;

            await inUseInterface.GiveBack();
        }
        #endregion

        #region INavigationService
        public override void OnNavigatedFrom(INavigationParameters parameters)
        {
            
        }

        public override void OnNavigatedTo(INavigationParameters parameters)
        {
            MachineItem = parameters["instance"] as Machine;

            IsBusy = true;
            Task.Run(LoadData);
        }
        #endregion
    }
}
